from plone.app.testing import PloneSandboxLayer
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting
from plone.app.testing import applyProfile
from plone.app.testing import quickInstallProduct

from plone.testing import z2
from Testing import ZopeTestCase


class TestLayer(PloneSandboxLayer):

    default_bases = (PLONE_FIXTURE,)

    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def setUpZope(self, app, configurationContext):

        # Set up sessioning objects
        app.REQUEST['SESSION'] = self.Session()
        ZopeTestCase.utils.setupCoreSessions(app)

        import seantis.dir.roadworks
        self.loadZCML(package=seantis.dir.roadworks)

    def setUpPloneSite(self, portal):
        quickInstallProduct(portal, 'seantis.dir.roadworks')
        applyProfile(portal, 'seantis.dir.roadworks:default')

    def tearDownZope(self, app):
        z2.uninstallProduct(app, 'seantis.dir.roadworks')


TESTFIXTURE = TestLayer()

INTEGRATION_TESTING = IntegrationTesting(
    bases=(TESTFIXTURE, ),
    name="Testfixture:Integration"
)

FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(TESTFIXTURE, ),
    name="Testfixture:Functional"
)



from unittest import TestCase
from os import path
from zope.interface import Invalid

from plone import api
from plone.namedfile import NamedFile
from plone.namedfile import NamedImage

from seantis.dir.base.interfaces import IMapMarker
from seantis.dir.roadworks.item import validate_image
from seantis.dir.roadworks.item import View
from seantis.dir.roadworks.tests import IntegrationTestCase


class TestImageValidator(TestCase):

    def test_validate_image(self):
        result = validate_image(None)
        self.assertEqual(None, result)

        image = NamedFile('dummy test data', filename=u'test.txt')
        self.assertRaises(Invalid, validate_image, image)

        image_path = path.join(path.dirname(__file__), 'data', 'test.png')
        image = open(image_path)
        image_data = image.read()
        image.close()
        image = NamedImage(image_data, filename=u'test.png')
        self.assertEqual(None, result)


class TestView(IntegrationTestCase):

    def test_details(self):
        self.login('admin')

        directory = api.content.create(
            container=self.new_temporary_folder(),
            type='seantis.dir.roadworks.directory',
            title='directory'
        )
        item = api.content.create(
            container=directory,
            type='seantis.dir.roadworks.item',
            title='item'
        )

        view = View(item, self.portal.REQUEST)
        self.assertEqual([], view.details())

        item.road = u'Unter der Egg'
        self.assertEqual([(u'Road', u'Unter der Egg')], view.details())

    def test_details_attachment(self):
        self.login('admin')

        directory = api.content.create(
            container=self.new_temporary_folder(),
            type='seantis.dir.roadworks.directory',
            title='directory'
        )
        item = api.content.create(
            container=directory,
            type='seantis.dir.roadworks.item',
            title='item'
        )

        item.road = u'Unter der Egg'
        item.attachment = NamedFile('Dummy content', filename=u'test.txt')

        view = View(item, self.portal.REQUEST)
        attachment_detail = view.details()[1]
        self.assertEqual(u'Attachment', attachment_detail[0])
        self.assertTrue(u'test.txt (0 KB)' in attachment_detail[1])


class TestRoadworkMapMarker(IntegrationTestCase):

    def test_url(self):
        self.login('admin')

        directory = api.content.create(
            container=self.new_temporary_folder(),
            type='seantis.dir.roadworks.directory',
            title='directory'
        )
        item = api.content.create(
            container=directory,
            type='seantis.dir.roadworks.item',
            title='item'
        )

        marker = IMapMarker(item)
        url = marker.url('A')
        expected = \
            '++resource++seantis.dir.roadworks.images/construction-marker.png'
        self.assertIn(expected, url)



import imghdr
from five import grok
from zope.schema import TextLine
from zope.schema.interfaces import IText
from zope.interface import Invalid
from plone.namedfile.field import NamedFile
from plone.namedfile.field import NamedImage
from plone.directives import form
from collective.dexteritytextindexer import searchable

from seantis.dir.base import item
from seantis.dir.base import core
from seantis.dir.base import utils
from seantis.dir.base.interfaces import (
    IFieldMapExtender, IDirectoryItem, IMapMarker
)

from seantis.dir.roadworks.directory import IRoadworksDirectory
from seantis.dir.roadworks import _


class IRoadworksDirectoryItem(IDirectoryItem):
    """Extends the seantis.dir.IDirectoryItem."""

    searchable('project')
    project = TextLine(
        title=_(u'Project'),
        required=False
    )

    searchable('section')
    section = TextLine(
        title=_(u'Section'),
        required=False
    )

    searchable('road')
    road = TextLine(
        title=_(u'Road'),
        required=False
    )

    searchable('works')
    works = TextLine(
        title=_(u'Construction Works'),
        required=False
    )

    searchable('timespan')
    timespan = TextLine(
        title=_(u'Timespan'),
        required=False
    )

    searchable('obstacles')
    obstacles = TextLine(
        title=_(u'Obstacles'),
        required=False
    )

    searchable('closures')
    closures = TextLine(
        title=_(u'Closures'),
        required=False
    )

    searchable('constructor')
    constructor = TextLine(
        title=_(u'Constructor'),
        required=False
    )

    searchable('contact')
    contact = TextLine(
        title=_(u'Contact'),
        required=False
    )

    image = NamedImage(
        title=_(u'Image'),
        required=False,
        default=None
    )

    attachment = NamedFile(
        title=_(u'Attachment'),
        required=False
    )


# Ensure that the uploaded image at least has an image header.
@form.validator(field=IRoadworksDirectoryItem['image'])
def validate_image(value):
    if not value:
        return

    if not imghdr.what(value.filename, value.data):
        raise Invalid(_(u'Unknown image format'))


class RoadworksDirectoryItem(item.DirectoryItem):
    pass


class View(core.View):
    """Default view of a seantis.dir.roadworks item."""
    grok.context(IRoadworksDirectoryItem)
    grok.require('zope2.View')

    template = grok.PageTemplateFile('templates/item.pt')

    def details(self):
        fields = IRoadworksDirectoryItem.names()

        def title(field):
            return utils.translate(
                self.context, self.request,
                IRoadworksDirectoryItem[field].title
            )

        titles = dict(zip(fields, map(title, fields)))

        items = []
        order = lambda f: IRoadworksDirectoryItem.get(f).order
        for field in sorted(fields, key=order):
            # Show only fields whose string representation makes sense.
            if (IText.providedBy(IRoadworksDirectoryItem[field]) and
                    getattr(self.context, field)):
                items.append((titles[field], getattr(self.context, field)))

        if self.context.attachment:
            attachment = self.context.attachment
            link = '<a href="%s/@@download/attachment">%s (%s KB)</a>' % (
                self.context.absolute_url(),
                attachment.filename,
                attachment.getSize() / 1024
            )
            items.append((titles['attachment'], link))

        return items


class ExtendedDirectoryItemFieldMap(grok.Adapter):
    """Adapter extending the import/export fieldmap of
    seantis.dir.roadworks.item.

    """
    grok.context(IRoadworksDirectory)
    grok.provides(IFieldMapExtender)

    def __init__(self, context):
        self.context = context

    def extend_import(self, itemmap):
        itemmap.typename = 'seantis.dir.roadworks.item'
        itemmap.interface = IRoadworksDirectoryItem

        extended = [
            'project',
            'road',
            'section',
            'works',
            'timespan',
            'obstacles',
            'closures',
            'constructor',
            'contact'
        ]

        itemmap.add_fields(extended, len(itemmap))


class RoadworkMapMarker(grok.Adapter):

    grok.context(IRoadworksDirectoryItem)
    grok.implements(IMapMarker)

    def url(self, letter):
        """
        Returns URL to a general marker image.
        """
        baseurl = self.context.absolute_url()
        imagedir = "/++resource++seantis.dir.roadworks.images"
        image = "/construction-marker.png"
        return baseurl + imagedir + image



from five import grok
from plone.namedfile.field import NamedImage

from seantis.dir.base import directory
from seantis.dir.base.interfaces import IDirectory
from seantis.dir.roadworks import _


class IRoadworksDirectory(IDirectory):
    """Extends the seantis.dir.base.directory.IDirectory"""

    image = NamedImage(
        title=_(u'Image'),
        required=False,
        default=None
    )


class RoadworksDirectory(directory.Directory):
    pass


class ExtendedDirectoryViewlet(grok.Viewlet):
    grok.context(IRoadworksDirectory)
    grok.name('seantis.dir.roadworks.directory.detail')
    grok.require('zope2.View')
    grok.viewletmanager(directory.DirectoryViewletManager)

    template = grok.PageTemplateFile('templates/directorydetail.pt')


class View(directory.View):
    grok.context(IRoadworksDirectory)
    grok.require('zope2.View')

    template = grok.PageTemplateFile('templates/directory.pt')

    itemsperpage = 100



from Products.CMFCore.utils import getToolByName

from seantis.dir.roadworks.directory import IRoadworksDirectory
from seantis.dir.roadworks.item import IRoadworksDirectoryItem
from seantis.dir.base.upgrades import (
    add_behavior_to_item,
    reset_images_and_attachments
)


def upgrade_to_1000(context):

    # add the new index first as the next step also does a reindex of all items
    setup = getToolByName(context, 'portal_setup')
    setup.runImportStepFromProfile(
        'profile-seantis.dir.roadworks:default', 'catalog'
    )

    reset_images_and_attachments(
        context, (IRoadworksDirectory, IRoadworksDirectoryItem)
    )

    add_behavior_to_item(
        context, 'seantis.dir.roadworks', IRoadworksDirectoryItem
    )


def upgrade_1000_to_1001(context):
    # add collective.geo.behaviour
    setup = getToolByName(context, 'portal_setup')
    setup.runAllImportStepsFromProfile(
        'profile-collective.geo.behaviour:default'
    )

    add_behavior_to_item(
        context, 'seantis.dir.roadworks', IRoadworksDirectoryItem
    )

    # update css and js
    getToolByName(context, 'portal_css').cookResources()
    getToolByName(context, 'portal_javascripts').cookResources()


def upgrade_1001_to_1002(context):
    # update types
    setup = getToolByName(context, 'portal_setup')
    setup.runImportStepFromProfile(
        'profile-seantis.dir.roadworks:default', 'typeinfo'
    )



from zope.security.management import newInteraction, endInteraction

from seantis.plonetools.testing import TestCase
from seantis.dir.roadworks.testing import (
    INTEGRATION_TESTING,
    FUNCTIONAL_TESTING
)


# to use with integration where security interactions need to be done manually
class IntegrationTestCase(TestCase):
    layer = INTEGRATION_TESTING

    def setUp(self):
        super(IntegrationTestCase, self).setUp()
        newInteraction()

    def tearDown(self):
        endInteraction()
        super(IntegrationTestCase, self).tearDown()


# to use with the browser which does its own security interactions
class FunctionalTestCase(TestCase):
    layer = FUNCTIONAL_TESTING



from setuptools import setup, find_packages
import os

version = '1.7.3'

tests_require = [
    'collective.testcaselayer',
    'plone.app.testing',
    'collective.betterbrowser[pyquery]'
]

setup(name='seantis.dir.roadworks',
      version=version,
      description="Directory of Roadworks",
      long_description='\n'.join((
          open("README.rst").read(),
          open(os.path.join("docs", "HISTORY.txt")).read()
      )),
      # Get more strings from
      # http://pypi.python.org/pypi?:action=list_classifiers
      classifiers=[
          "Framework :: Plone",
          "Programming Language :: Python",
      ],
      keywords='',
      author='Seantis GmbH',
      author_email='info@seantis.ch',
      url='https://github.com/seantis/seantis.dir.roadworks',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['seantis', 'seantis.dir'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'Plone',
          'plone.app.dexterity',
          'collective.autopermission',
          'collective.testcaselayer',
          'collective.dexteritytextindexer',
          'seantis.dir.base>=1.7',
          'seantis.plonetools>=0.9'
      ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )
